@@ -503,7 +503,6 @@ do_draw_gdiplus(LPGW lpgw, Graphics &graphics, LPRECT rect, enum draw_target tar
 	POINT corners[4];			/* image corners */
 	int color_mode = 0;			/* image color mode */
 
-#ifdef EAM_BOXED_TEXT
 	struct s_boxedtext {
 		TBOOLEAN boxing;
 		t_textbox_options option;
@@ -512,7 +511,6 @@ do_draw_gdiplus(LPGW lpgw, Graphics &graphics, LPRECT rect, enum draw_target tar
 		RECT  box;
 		int   angle;
 	} boxedtext;
-#endif
 
 	/* point symbols */
 	enum win_pointtypes last_symbol = W_invalid_pointtype;
@@ -614,9 +612,7 @@ do_draw_gdiplus(LPGW lpgw, Graphics &graphics, LPRECT rect, enum draw_target tar
 		lpgw->keyboxes[i].top = 0;
 	}
 
-#ifdef EAM_BOXED_TEXT
 	boxedtext.boxing = FALSE;
-#endif
 
 	/* do the drawing */
 	blkptr = lpgw->gwopblk_head;
@@ -782,6 +778,7 @@ do_draw_gdiplus(LPGW lpgw, Graphics &graphics, LPRECT rect, enum draw_target tar
 
 		case W_polyline: {
 			POINTL * poly = (POINTL *) LocalLock(curptr->htext);
+			if (poly == NULL) break; // memory allocation failed
 			polyi = curptr->x;
 			PointF * points = new PointF[polyi];
 			for (int i = 0; i < polyi; i++) {
@@ -874,6 +871,7 @@ do_draw_gdiplus(LPGW lpgw, Graphics &graphics, LPRECT rect, enum draw_target tar
 				gdiplusSetDashStyle(&pen, static_cast<DashStyle>(cur_penstruct.lopnStyle));
 			} else if (dt == DASHTYPE_CUSTOM) {
 				t_dashtype * dash = static_cast<t_dashtype *>(LocalLock(curptr->htext));
+				if (dash == NULL) break; // memory allocation failed
 				INT count = 0;
 				while ((dash->pattern[count] != 0.) && (count < DASHPATTERN_LENGTH)) count++;
 				pen.SetDashPattern(dash->pattern, count);
@@ -904,11 +902,7 @@ do_draw_gdiplus(LPGW lpgw, Graphics &graphics, LPRECT rect, enum draw_target tar
 					}
 					RectF size;
 					int dxl, dxr;
-#ifndef EAM_BOXED_TEXT
-					if (keysample) {
-#else
 					if (keysample || boxedtext.boxing) {
-#endif
 						graphics.MeasureString(textw, -1, font, PointF(0,0), &stringFormat, &size);
 						if (lpgw->justify == LEFT) {
 							dxl = 0;
@@ -924,7 +918,6 @@ do_draw_gdiplus(LPGW lpgw, Graphics &graphics, LPRECT rect, enum draw_target tar
 						draw_update_keybox(lpgw, plotno, xdash - dxl, ydash - size.Height / 2);
 						draw_update_keybox(lpgw, plotno, xdash + dxr, ydash + size.Height / 2);
 					}
-#ifdef EAM_BOXED_TEXT
 					if (boxedtext.boxing) {
 						if (boxedtext.box.left > (xdash - boxedtext.start.x - dxl))
 							boxedtext.box.left = xdash - boxedtext.start.x - dxl;
@@ -937,7 +930,6 @@ do_draw_gdiplus(LPGW lpgw, Graphics &graphics, LPRECT rect, enum draw_target tar
 						/* We have to remember the text angle as well. */
 						boxedtext.angle = lpgw->angle;
 					}
-#endif
 					free(textw);
 				}
 			}
@@ -957,7 +949,6 @@ do_draw_gdiplus(LPGW lpgw, Graphics &graphics, LPRECT rect, enum draw_target tar
 					draw_update_keybox(lpgw, plotno, xdash - extend.left, ydash - extend.top);
 					draw_update_keybox(lpgw, plotno, xdash + extend.right, ydash + extend.bottom);
 				}
-#ifdef EAM_BOXED_TEXT
 				if (boxedtext.boxing) {
 					if (boxedtext.box.left > (boxedtext.start.x - xdash - extend.left))
 						boxedtext.box.left = boxedtext.start.x - xdash - extend.left;
@@ -970,7 +961,6 @@ do_draw_gdiplus(LPGW lpgw, Graphics &graphics, LPRECT rect, enum draw_target tar
 					/* We have to store the text angle as well. */
 					boxedtext.angle = lpgw->angle;
 				}
-#endif
 			}
 			LocalUnlock(curptr->htext);
 			break;
@@ -987,7 +977,6 @@ do_draw_gdiplus(LPGW lpgw, Graphics &graphics, LPRECT rect, enum draw_target tar
 			}
 			break;
 
-#ifdef EAM_BOXED_TEXT
 		case W_boxedtext:
 			if (seq == 0) {
 				boxedtext.option = (t_textbox_options) curptr->x;
@@ -1099,7 +1088,6 @@ do_draw_gdiplus(LPGW lpgw, Graphics &graphics, LPRECT rect, enum draw_target tar
 				break;
 			}
 			break;
-#endif
 
 		case W_fillstyle: {
 			/* HBB 20010916: new entry, needed to squeeze the many
@@ -1419,6 +1407,7 @@ do_draw_gdiplus(LPGW lpgw, Graphics &graphics, LPRECT rect, enum draw_target tar
 			} else {
 				/* The last OP contains the image and it's size */
 				char * image = (char *) LocalLock(curptr->htext);
+				if (image == NULL) break; // memory allocation failed
 				unsigned int width = curptr->x;
 				unsigned int height = curptr->y;
 				if (image) {
